@@ -77,8 +77,6 @@ typedef struct _LoadContext {
     SeahorseSSHSource *ssrc;
     GHashTable *loaded;
     GHashTable *checks;
-    GQuark match;
-    gboolean matched;
     gchar *pubfile;
     gchar *privfile;
 } LoadContext;
@@ -88,7 +86,10 @@ typedef struct _ImportContext {
     SeahorseMultiOperation *mop;
 } ImportContext;
 
-G_DEFINE_TYPE (SeahorseSSHSource, seahorse_ssh_source, SEAHORSE_TYPE_SOURCE);
+static void seahorse_source_iface (SeahorseSourceIface *iface);
+
+G_DEFINE_TYPE_EXTENDED (SeahorseSSHSource, seahorse_ssh_source, G_TYPE_OBJECT, 0,
+                        G_IMPLEMENT_INTERFACE (SEAHORSE_TYPE_SOURCE, seahorse_source_iface));
 
 #define AUTHORIZED_KEYS_FILE    "authorized_keys"
 #define OTHER_KEYS_FILE         "other_keys.seahorse"
@@ -141,24 +142,6 @@ check_file_for_ssh_private (SeahorseSSHSource *ssrc, const gchar *filename)
     return check_data_for_ssh_private (ssrc, buf);
 }
 
-static void
-key_changed (SeahorseKey *skey, SeahorseKeyChange change, SeahorseSource *sksrc)
-{
-    /* TODO: We need to fix these key change flags. Currently the only thing
-     * that sets 'ALL' is when we replace a key in an skey. We don't 
-     * need to reload in that case */
-    
-    if (change != SKEY_CHANGE_ALL)
-        seahorse_source_load_async (sksrc, seahorse_key_get_keyid (skey));
-}
-
-static void
-key_destroyed (SeahorseKey *skey, SeahorseSource *sksrc)
-{
-    g_signal_handlers_disconnect_by_func (skey, key_changed, sksrc);
-    g_signal_handlers_disconnect_by_func (skey, key_destroyed, sksrc);
-}
-
 static void
 cancel_scheduled_refresh (SeahorseSSHSource *ssrc)
 {
@@ -170,7 +153,7 @@ cancel_scheduled_refresh (SeahorseSSHSource *ssrc)
 }
 
 static void
-remove_key_from_context (gpointer hkey, SeahorseKey *dummy, SeahorseSSHSource *ssrc)
+remove_key_from_context (gpointer hkey, SeahorseObject *dummy, SeahorseSSHSource *ssrc)
 {
     GQuark keyid = GPOINTER_TO_UINT (hkey);
     SeahorseObject *sobj;
@@ -186,7 +169,7 @@ scheduled_refresh (SeahorseSSHSource *ssrc)
 {
     DEBUG_REFRESH ("scheduled refresh event ocurring now\n");
     cancel_scheduled_refresh (ssrc);
-    seahorse_source_load_async (SEAHORSE_SOURCE (ssrc), 0);
+    seahorse_source_load_async (SEAHORSE_SOURCE (ssrc));
     return FALSE; /* don't run again */    
 }
 
@@ -229,6 +212,7 @@ monitor_ssh_homedir (GFileMonitor *handle, GFile *file, GFile *other_file,
 	if (event_type != G_FILE_MONITOR_EVENT_DELETED && 
 	    !ends_with (path, AUTHORIZED_KEYS_FILE) &&
 	    !ends_with (path, OTHER_KEYS_FILE) && 
+		!ends_with (path, ".pub") &&
 	    !check_file_for_ssh_private (ssrc, path)) {
 		g_free (path);
 		return;
@@ -271,7 +255,7 @@ ssh_key_from_data (SeahorseSSHSource *ssrc, LoadContext *ctx,
     }
 
     /* Make sure it's valid */
-    keyid = seahorse_ssh_key_get_cannonical_id (keydata->fingerprint);
+    keyid = seahorse_ssh_key_calc_cannonical_id (keydata->fingerprint);
     g_return_val_if_fail (keyid, NULL);
 
     /* Does this key exist in the context? */
@@ -301,16 +285,7 @@ ssh_key_from_data (SeahorseSSHSource *ssrc, LoadContext *ctx,
         g_hash_table_insert (ctx->loaded, GUINT_TO_POINTER (keyid), 
                                           GUINT_TO_POINTER (TRUE));
     }
-
-    /* Looking for a specific key */
-    if (ctx->match) {
-        if(ctx->match != keyid) {
-            seahorse_ssh_key_data_free (keydata);
-            return NULL;
-        }
-        ctx->matched = TRUE;
-    }
-        
+     
     /* If we already have this key then just transfer ownership of keydata */
     if (prev) {
         g_object_set (prev, "key-data", keydata, NULL);
@@ -320,11 +295,7 @@ ssh_key_from_data (SeahorseSSHSource *ssrc, LoadContext *ctx,
     /* Create a new key */        
     g_assert (keydata);
     skey = seahorse_ssh_key_new (sksrc, keydata);
-            
-    /* We listen in to get notified of changes on this key */
-    g_signal_connect (skey, "changed", G_CALLBACK (key_changed), sksrc);
-    g_signal_connect (skey, "destroy", G_CALLBACK (key_destroyed), sksrc);
-        
+
     seahorse_context_take_object (SCTX_APP (), SEAHORSE_OBJECT (skey));
     return skey;
 }
@@ -344,9 +315,6 @@ parsed_authorized_key (SeahorseSSHKeyData *data, gpointer arg)
     /* Check and register thet key with the context, frees keydata */
     ssh_key_from_data (ctx->ssrc, ctx, data);
     
-    if (ctx->matched)
-        return FALSE;
-    
     return TRUE;
 }
 
@@ -365,9 +333,6 @@ parsed_other_key (SeahorseSSHKeyData *data, gpointer arg)
     /* Check and register thet key with the context, frees keydata */
     ssh_key_from_data (ctx->ssrc, ctx, data);
     
-    if (ctx->matched)
-        return FALSE;
-    
     return TRUE;
 }
 
@@ -386,9 +351,6 @@ parsed_public_key (SeahorseSSHKeyData *data, gpointer arg)
     /* Check and register thet key with the context, frees keydata */
     ssh_key_from_data (ctx->ssrc, ctx, data);
     
-    if (ctx->matched)
-        return FALSE;
-    
     return TRUE;
 }
 
@@ -445,7 +407,7 @@ import_private_key (SeahorseSSHSecData *data, gpointer arg)
  */
 
 static SeahorseOperation*
-seahorse_ssh_source_load (SeahorseSource *sksrc, GQuark keymatch)
+seahorse_ssh_source_load (SeahorseSource *sksrc)
 {
     SeahorseSSHSource *ssrc;
     GError *err = NULL;
@@ -467,16 +429,13 @@ seahorse_ssh_source_load (SeahorseSource *sksrc, GQuark keymatch)
         return seahorse_operation_new_complete (err);
 
     memset (&ctx, 0, sizeof (ctx));
-    ctx.match = keymatch;
-    ctx.matched = FALSE;
     ctx.ssrc = ssrc;
     
     /* Since we can find duplicate keys, limit them with this hash */
     ctx.loaded = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, NULL);
     
     /* Keys that currently exist, so we can remove any that disappeared */
-    if (!keymatch)
-        ctx.checks = load_present_keys (sksrc);
+    ctx.checks = load_present_keys (sksrc);
 
     /* For each private key file */
     for(;;) {
@@ -502,60 +461,47 @@ seahorse_ssh_source_load (SeahorseSource *sksrc, GQuark keymatch)
         g_free (ctx.privfile);
         g_free (ctx.pubfile);
         ctx.privfile = ctx.pubfile = NULL;
-
-        if (ctx.matched)
-            break;
     }
 
     g_dir_close (dir);
     
     /* Now load the authorized file */
-    if (!ctx.matched) {
+    ctx.privfile = NULL;
+    ctx.pubfile = seahorse_ssh_source_file_for_public (ssrc, TRUE);
         
-        ctx.privfile = NULL;
-        ctx.pubfile = seahorse_ssh_source_file_for_public (ssrc, TRUE);
-        
-        if (g_file_test (ctx.pubfile, G_FILE_TEST_EXISTS)) {
-            seahorse_ssh_key_data_parse_file (ctx.pubfile, parsed_authorized_key, NULL, &ctx, &err);
-            if (err != NULL) {
-                g_warning ("couldn't read SSH file: %s (%s)", ctx.pubfile, err->message);
-                g_clear_error (&err);
-            }
+    if (g_file_test (ctx.pubfile, G_FILE_TEST_EXISTS)) {
+        seahorse_ssh_key_data_parse_file (ctx.pubfile, parsed_authorized_key, NULL, &ctx, &err);
+        if (err != NULL) {
+            g_warning ("couldn't read SSH file: %s (%s)", ctx.pubfile, err->message);
+            g_clear_error (&err);
         }
-        
-        g_free (ctx.pubfile);
-		ctx.pubfile = NULL;
     }
+        
+    g_free (ctx.pubfile);
+    ctx.pubfile = NULL;
     
     /* Load the other keys file */
-    if (!ctx.matched) {
-        
-        ctx.privfile = NULL;
-        ctx.pubfile = seahorse_ssh_source_file_for_public (ssrc, FALSE);
-        if (g_file_test (ctx.pubfile, G_FILE_TEST_EXISTS)) {
-            seahorse_ssh_key_data_parse_file (ctx.pubfile, parsed_other_key, NULL, &ctx, &err);
-            if (err != NULL) {
-                g_warning ("couldn't read SSH file: %s (%s)", ctx.pubfile, err->message);
-                g_clear_error (&err);
-            }
+    ctx.privfile = NULL;
+    ctx.pubfile = seahorse_ssh_source_file_for_public (ssrc, FALSE);
+    if (g_file_test (ctx.pubfile, G_FILE_TEST_EXISTS)) {
+        seahorse_ssh_key_data_parse_file (ctx.pubfile, parsed_other_key, NULL, &ctx, &err);
+        if (err != NULL) {
+            g_warning ("couldn't read SSH file: %s (%s)", ctx.pubfile, err->message);
+            g_clear_error (&err);
         }
-        
-        g_free (ctx.pubfile);
-		ctx.pubfile = NULL;
     }
+        
+    g_free (ctx.pubfile);
+    ctx.pubfile = NULL;
 
     /* Clean up and done */
     if (ctx.checks) {
-        g_assert (!ctx.matched);
         g_hash_table_foreach (ctx.checks, (GHFunc)remove_key_from_context, ssrc);
         g_hash_table_destroy (ctx.checks);
     }
     
     g_hash_table_destroy (ctx.loaded);
 
-    /* Just a double check of our logic */
-    g_assert (!ctx.matched || keymatch);
-    
     return seahorse_operation_new_complete (NULL);
 }
 
@@ -592,7 +538,7 @@ seahorse_ssh_source_export (SeahorseSource *sksrc, GList *keys,
     gchar *results = NULL;
     gchar *raw = NULL;
     GError *error = NULL;
-    SeahorseKey *skey;
+    SeahorseObject *object;
     GList *l;
     gsize written;
     
@@ -600,15 +546,15 @@ seahorse_ssh_source_export (SeahorseSource *sksrc, GList *keys,
     g_return_val_if_fail (G_IS_OUTPUT_STREAM (output), NULL);
     
     for (l = keys; l; l = g_list_next (l)) {
-        skey = SEAHORSE_KEY (l->data);
+        object = SEAHORSE_OBJECT (l->data);
         
-        g_assert (SEAHORSE_IS_SSH_KEY (skey));
+        g_assert (SEAHORSE_IS_SSH_KEY (object));
         
         results = NULL;
         raw = NULL;
         
         keydata = NULL;
-        g_object_get (skey, "key-data", &keydata, NULL);
+        g_object_get (object, "key-data", &keydata, NULL);
         g_return_val_if_fail (keydata, NULL);
         
         /* We should already have the data loaded */
@@ -649,62 +595,6 @@ seahorse_ssh_source_export (SeahorseSource *sksrc, GList *keys,
     	return op;
 }
 
-static SeahorseOperation*
-seahorse_ssh_source_remove (SeahorseSource *sksrc, SeahorseObject *sobj)
-{
-    SeahorseSSHKeyData *keydata = NULL;
-    SeahorseSSHSource *ssrc = SEAHORSE_SSH_SOURCE (sksrc);
-    gboolean ret = TRUE;
-    GError *err = NULL;
-    gchar *fullpath;
-    
-    g_return_val_if_fail (SEAHORSE_IS_SSH_SOURCE (sksrc), NULL);
-    g_return_val_if_fail (SEAHORSE_IS_SSH_KEY (sobj), NULL);
-    g_return_val_if_fail (seahorse_object_get_source (sobj) == sksrc, NULL);
-
-    g_object_get (sobj, "key-data", &keydata, NULL);
-    g_return_val_if_fail (keydata, FALSE);
-    
-    /* Just part of a file for this key */
-    if (keydata->partial) {
-        
-        /* Take just that line out of the file */
-        if (keydata->pubfile) {
-            seahorse_ssh_key_data_filter_file (keydata->pubfile, NULL, keydata, &err);
-        }
-        
-        /* Make sure to take it out of any other files too */
-        if (ret && keydata->authorized) {
-            fullpath = seahorse_ssh_source_file_for_public (ssrc, FALSE);
-            seahorse_ssh_key_data_filter_file (fullpath, NULL, keydata, &err);
-            g_free (fullpath);
-        }
-        
-        
-    /* A full full for this key */
-    } else {
-        
-        if (keydata->pubfile) {
-            if (g_unlink (keydata->pubfile) == -1) {
-                g_set_error (&err, G_FILE_ERROR, g_file_error_from_errno (errno), 
-                             "%s", g_strerror (errno));
-            }
-        }
-
-        if (ret && keydata->privfile) {
-            if (g_unlink (keydata->privfile) == -1) {
-                g_set_error (&err, G_FILE_ERROR, g_file_error_from_errno (errno), 
-                             "%s", g_strerror (errno));
-            }
-        }
-    }
-
-    if (err == NULL)
-        seahorse_context_remove_object (SCTX_APP (), sobj);
-
-    return seahorse_operation_new_complete (err);
-}
-
 static void 
 seahorse_ssh_source_set_property (GObject *object, guint prop_id, const GValue *value, 
                                   GParamSpec *pspec)
@@ -726,7 +616,7 @@ seahorse_ssh_source_get_property (GObject *object, guint prop_id, GValue *value,
         g_value_set_string (value, _("Secure Shell Key"));
         break;
     case PROP_LOCATION:
-        g_value_set_uint (value, SEAHORSE_LOCATION_LOCAL);
+        g_value_set_enum (value, SEAHORSE_LOCATION_LOCAL);
         break;
     case PROP_BASE_DIRECTORY:
         g_value_set_string (value, ssrc->priv->ssh_homedir);
@@ -805,7 +695,6 @@ static void
 seahorse_ssh_source_class_init (SeahorseSSHSourceClass *klass)
 {
     GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
-    SeahorseSourceClass *parent_class = SEAHORSE_SOURCE_CLASS (klass);
    
     seahorse_ssh_source_parent_class = g_type_class_peek_parent (klass);
     
@@ -814,29 +703,25 @@ seahorse_ssh_source_class_init (SeahorseSSHSourceClass *klass)
     gobject_class->set_property = seahorse_ssh_source_set_property;
     gobject_class->get_property = seahorse_ssh_source_get_property;
     
-    parent_class->canonize_id = seahorse_ssh_key_get_cannonical_id;
-    parent_class->load = seahorse_ssh_source_load;
-    parent_class->import = seahorse_ssh_source_import;
-    parent_class->export = seahorse_ssh_source_export;
-    parent_class->remove = seahorse_ssh_source_remove;
- 
-    g_object_class_install_property (gobject_class, PROP_KEY_TYPE,
-        g_param_spec_uint ("key-type", "Key Type", "Key type that originates from this key source.", 
-                           0, G_MAXUINT, SKEY_UNKNOWN, G_PARAM_READABLE));
+	g_object_class_override_property (gobject_class, PROP_KEY_TYPE, "key-type");
+	g_object_class_override_property (gobject_class, PROP_KEY_DESC, "key-desc");
+	g_object_class_override_property (gobject_class, PROP_LOCATION, "location");
 
-    g_object_class_install_property (gobject_class, PROP_KEY_DESC,
-        g_param_spec_string ("key-desc", "Key Desc", "Description for keys that originate here.",
-                             NULL, G_PARAM_READABLE));
-
-    g_object_class_install_property (gobject_class, PROP_LOCATION,
-        g_param_spec_uint ("location", "Key Location", "Where the key is stored. See SeahorseLocation", 
-                           0, G_MAXUINT, SEAHORSE_LOCATION_INVALID, G_PARAM_READABLE));    
-                           
     g_object_class_install_property (gobject_class, PROP_BASE_DIRECTORY,
         g_param_spec_string ("base-directory", "Key directory", "Directory where the keys are stored",
                              NULL, G_PARAM_READABLE));
     
-	seahorse_registry_register_type (NULL, SEAHORSE_TYPE_SSH_SOURCE, "key-source", "local", SEAHORSE_SSH_STR, NULL);
+	seahorse_registry_register_type (NULL, SEAHORSE_TYPE_SSH_SOURCE, "source", "local", SEAHORSE_SSH_STR, NULL);
+	
+	seahorse_registry_register_function (NULL, seahorse_ssh_key_calc_cannonical_id, "canonize", SEAHORSE_SSH_STR, NULL);
+}
+
+static void 
+seahorse_source_iface (SeahorseSourceIface *iface)
+{
+	iface->load = seahorse_ssh_source_load;
+	iface->import = seahorse_ssh_source_import;
+	iface->export = seahorse_ssh_source_export;
 }
 
 /* -----------------------------------------------------------------------------
@@ -875,7 +760,7 @@ seahorse_ssh_source_key_for_filename (SeahorseSSHSource *ssrc, const gchar *priv
         
         /* Force loading of all new keys */
         if (!i) {
-            seahorse_source_load_sync (SEAHORSE_SOURCE (ssrc), 0);
+            seahorse_source_load_sync (SEAHORSE_SOURCE (ssrc));
         }
     }
     
